# -*- coding: utf-8 -*-
"""
Created on Tue Apr  7 12:40:46 2015

@author: adrienkuntz
"""

cpdef double Wki(double chi)
cpdef double Wgi(double chi)
cpdef double redshifti(double chi)
cdef double chiCMB