# -*- coding: utf-8 -*-
"""
Created on Wed Apr  1 14:04:31 2015

@author: adrienkuntz
"""

cpdef size_t N
cdef double *kptr
cdef double *Pptr

cpdef double P(double k, double z)
cdef double norme2(double k1, double k2, double u12)
cdef double norme3(double k1, double k2, double k3, double u12, double u13, double u23)
cdef double F2(double k1, double k2, double u12)
cdef double BPT(double k1, double k2, double k3, double u12, double u13, double u23, double z)
cdef double G2(double k1, double k2, double u12)
cdef double F3(double k1, double k2, double k3, double u12, double u13, double u23)
cdef double F3s(double k1, double k2, double k3, double u12, double u13, double u23)
cdef double TPT(double k1, double k2, double u12, double z)